(function () {
	'use strict';

	var templateString = "\n<%if (typeof lazysizesBlurhash !== 'undefined') {%>\n<span class=\"setting lazysizes-blurhash\">\n\t<span class=\"name\">Lazysizes Blurhash</span>\n\t<span class=\"value\" <%= !lazysizesBlurhash ? 'style=\"padding-top: 0;\"' : '' %>>\n\t\t<%if (!lazysizesBlurhash) {%>\n\t\t\t<span>\n\t\t\t\t<%= lazysizesStrings.notGenerated %>\n\t\t\t</span>\n\t\t\t<button type=\"button\" class=\"button button-primary lazysizes-blurhash-generate\" <%= lazysizesLoading ? 'disabled' : '' %>><%= lazysizesStrings.generate %></button>\n\t\t<%} else {%>\n\t\t\t<div style=\"padding-bottom: 8px;\">\n\t\t\t\t<%= lazysizesStrings.current + lazysizesBlurhash %>\n\t\t\t</div>\n\t\t\t<button type=\"button\" class=\"button lazysizes-blurhash-delete\" <%= lazysizesLoading ? 'disabled' : '' %>><%= lazysizesStrings.delete %></button>\n\t\t<%}%>\n\t\t<span class=\"spinner <%= lazysizesLoading ? 'is-active' : '' %>\" style=\"padding-top: 0; float: none; min-width: 20px;\"></span>\n\t\t<%if (lazysizesError) {%>\n\t\t\t<div>\n\t\t\t\t<%= lazysizesError %>\n\t\t\t</div>\n\t\t<%}%>\n\t</span>\n</span>\n<p class=\"description\">\n\t<%= lazysizesStrings.description %>\n</p>\n<%}%>\n";

	var templateFunction = _.template(templateString);

	var clickFunction = function clickFunction(e) {
	  var _this = this;

	  var action = '';

	  if (e.target.classList.contains('lazysizes-blurhash-generate')) {
	    action = 'generate';
	  } else if (e.target.classList.contains('lazysizes-blurhash-delete')) {
	    action = 'delete';
	  } else {
	    return;
	  }

	  this.model.set('lazysizesLoading', true);
	  lazysizesAjax(action, this.model.attributes.id, this.model.attributes.nonces.lazysizes[action], function (response, status, errorCode) {
	    _this.model.set('lazysizesLoading', false);

	    if (status === 'error') {
	      _this.model.set('lazysizesError', "".concat(lazysizesStrings.error, " (").concat(errorCode, ")"));
	    } else {
	      if (response.success) {
	        if (action === 'generate') {
	          _this.model.set('lazysizesBlurhash', response.blurhash);
	        } else if (action === 'delete') {
	          _this.model.set('lazysizesBlurhash', false);
	        }
	      } else {
	        _this.model.set('lazysizesError', response.data[0].message);
	      }
	    }
	  });
	}; // Based on code by Thomas Griffin.
	// See https://gist.github.com/sunnyratilal/5650341.
	// In Media Library.


	wp.media.view.Attachment.Details.TwoColumn = wp.media.view.Attachment.Details.TwoColumn.extend({
	  initialize: function initialize() {
	    wp.media.view.Attachment.Details.prototype.initialize.apply(this, arguments); // Always make sure that our content is up to date.

	    this.listenTo(this.model, 'change', this.render);
	  },
	  events: {
	    'click .setting.lazysizes-blurhash .button': clickFunction
	  },
	  render: function render() {
	    // Ensure that the main attachment fields are rendered.
	    wp.media.view.Attachment.prototype.render.apply(this, arguments);

	    if (this.model.changedAttributes(['lazysizesBlurhash', 'lazysizesError']) === false) {
	      this.model.fetch();
	    } // Detach the views, append our custom fields, make sure that our data is fully updated and re-render the updated view.


	    this.views.detach();

	    if (this.model.attributes.type === 'image') {
	      this.$el.find('.settings').append(templateFunction(this.model.toJSON()));
	    }

	    this.views.render();
	    return this;
	  }
	}); // In post editor, when selecting attachment.

	wp.media.view.Attachment.Details = wp.media.view.Attachment.Details.extend({
	  initialize: function initialize() {
	    // Always make sure that our content is up to date.
	    this.listenTo(this.model, 'change', this.render);
	  },
	  events: {
	    'click .setting.lazysizes-blurhash .button': clickFunction
	  },
	  render: function render() {
	    // Ensure that the main attachment fields are rendered.
	    wp.media.view.Attachment.prototype.render.apply(this, arguments);

	    if (this.model.changedAttributes(['lazysizesBlurhash', 'lazysizesError']) === false) {
	      this.model.fetch();
	    } // Detach the views, append our custom fields, make sure that our data is fully updated and re-render the updated view.


	    this.views.detach();

	    if (this.model.attributes.type === 'image') {
	      this.$el.append(templateFunction(this.model.toJSON()));
	    }

	    this.views.render();
	    return this;
	  }
	});

	function lazysizesAjax(action, attachmentId, nonce, callback) {
	  jQuery.ajax({
	    type: 'POST',
	    url: ajaxurl,
	    data: {
	      action: 'lazysizes_blurhash',
	      nonce: nonce,
	      mode: action,
	      attachmentId: attachmentId
	    },
	    success: callback,
	    error: callback
	  });
	}

}());
